// 005 u_gof_interpreter
// 11 dec 2004

// -- Delphi implementation from the book:
// --   Design Patterns Elements of Reusable Object-Oriented Software
// --   Addison Wesley - 1994 - ISBN 0-201-63361-2
// -- (C) Erich GAMMA, Richard HELM, Ralph JOHNSON, John VLISSIDES 1994

// -- translation from C to Delphi performed by
// --   Felix John COLIBRI  http://www.felix-colibri.com

(*$r+*)

unit u_gof_interpreter;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    Memo1: TMemo;
    exit_: TButton;
    clear_: TButton;
    interpret_: TButton;
    scan_: TButton;
    parse_: TButton;
    execute_: TButton;
    procedure FormCreate(Sender: TObject);
    procedure exit_Click(Sender: TObject);
    procedure clear_Click(Sender: TObject);
    procedure interpret_Click(Sender: TObject);
    procedure scan_Click(Sender: TObject);
    procedure parse_Click(Sender: TObject);
    procedure execute_Click(Sender: TObject);
  private

  public

  end;

    var Form1: TForm1;

  implementation
    uses u_types_constants, u_c_log, u_c_display
        , u_dir, u_file

        , u_c_scanner_15
        , u_c_ast_tree_15
        , u_c_instruction_list_15
        , u_c_variable_list_15
        , u_c_parser_15

        , u_c_interpreter_15

        , u_c_compiler_15
    ;

    {$R *.DFM}

 
 
    procedure TForm1.FormCreate(Sender: TObject);
      begin
        initialize_display(Memo1.Lines);
        initialize_default_log;
      end; // FormCreate

    procedure TForm1.exit_Click(Sender: TObject);
      begin
        Close;
      end; // close_Click

    procedure TForm1.clear_Click(Sender: TObject);
      begin
        clear_display;
      end; // clear_Click

    const k_source_name= '..\__data\assignments.txt';

    procedure TForm1.scan_Click(Sender: TObject);
      begin
        check_path_and_name(k_source_name);
        with c_scanner.create_scanner('scanner', k_source_name) do
        begin
          if f_load
            then begin
                test_scanner;
              end
            else display('could_not_find');
          Free;
        end; // with c_scanner
      end; // create_singleton_Click

    procedure TForm1.parse_Click(Sender: TObject);
      var l_c_scanner: c_scanner;
          l_c_parser: c_parser;
          l_c_instruction_list: c_instruction_list;
      begin
        l_c_scanner:= c_scanner.create_scanner('scanner', k_source_name);

        with l_c_scanner do
        begin
          if f_load
            then begin
                l_c_parser:= c_parser.create_parser('parser');

                l_c_instruction_list:= c_instruction_list.create_instruction_list('instr');
                l_c_parser.parse(l_c_scanner, l_c_instruction_list);
              end
            else display('could_not_find');

          Free;
        end; // with l_c_scanner
      end; // parse_Click

    procedure TForm1.execute_Click(Sender: TObject);
      var l_c_scanner: c_scanner;
          l_c_parser: c_parser;
          l_c_instruction_list: c_instruction_list;
          l_c_variable_list: c_variable_list;
          l_c_interpreter: c_interpreter;
      begin
        l_c_scanner:= c_scanner.create_scanner('scanner', k_source_name);

        with l_c_scanner do
        begin
          if f_load
            then begin
                l_c_parser:= c_parser.create_parser('parser');

                l_c_instruction_list:= c_instruction_list.create_instruction_list('instr');
                l_c_parser.parse(l_c_scanner, l_c_instruction_list);

                l_c_variable_list:= c_variable_list.create_variable_list('var');

                l_c_interpreter:= c_interpreter.create_interpreter;
                l_c_interpreter.interpret(l_c_instruction_list, l_c_variable_list);

                l_c_variable_list.display_variable_list;
              end
            else display('could_not_find');

          Free;
        end; // with l_c_scanner
      end; // execute_Click

    procedure TForm1.interpret_Click(Sender: TObject);
      begin
        with c_compiler.create_compiler do
        begin
          compile(k_source_name, Memo1.Lines);
        end;
      end; // interpret_Click

end.


