// 005 u_c_ast_tree_15
// 11 dec 2004

// -- Delphi implementation from the book:
// --   Design Patterns Elements of Reusable Object-Oriented Software
// --   Addison Wesley - 1994 - ISBN 0-201-63361-2
// -- (C) Erich GAMMA, Richard HELM, Ralph JOHNSON, John VLISSIDES 1994

// -- translation from C to Delphi performed by
// --   Felix John COLIBRI  http://www.felix-colibri.com

(*$r+*)

unit u_c_ast_tree_15;
  interface
    uses Classes, u_c_basic_object;

    type c_node= class(c_basic_object)
                   m_c_left, m_c_right: c_node;
                   m_id: Integer;

                   Constructor create_node(p_name: String);
                   procedure display_node;
                   function f_display: String; Virtual;
                 end;

         c_identifier_node= class(c_node)
                              Constructor create_identifier_node(p_name: String);
                            end; // c_identifier_node

         c_litteral_node= class(c_node)
                            Constructor create_litteral_node(p_name: String);
                          end; // c_litteral_node

         c_operation_node= class(c_node)
                            Constructor create_operation_node(p_name: String; p_c_left, p_c_right: c_node);
                          end; // c_operation_node

  implementation
    uses SysUtils, u_strings, u_c_display;

    // -- leaves

    var s_number: Integer= 0;

    Constructor c_node.create_node(p_name: String);
      begin
        InHerited create_basic_object(p_name);

        m_id:= s_number;
        Inc(s_number);
      end; // create_node

    function c_node.f_display: String;
      begin
        Result:= m_name;
      end; // f_display

    procedure c_node.display_node;

      procedure display_node_recursive(p_level: Integer; p_c_node: c_node);
        begin
          if p_c_node<> nil
            then
              with p_c_node do
              begin
                display_node_recursive(p_level+ 1, m_c_left);
                display(f_spaces(2* p_level)+ f_display);
                display_node_recursive(p_level+ 1, m_c_right);
              end;
        end; // display_node_recursive

      begin // display_node
        display_node_recursive(0, Self);
      end; // display_node

    // -- c_identifier_node

    Constructor c_identifier_node.create_identifier_node(p_name: String);
      begin
        Inherited create_node(p_name);
      end; // create_identifier_node

    // -- c_litteral_node

    Constructor c_litteral_node.create_litteral_node(p_name: String);
      begin
        Inherited create_node(p_name);
      end; // create_litteral_node

    // -- c_operation_node

    Constructor c_operation_node.create_operation_node(p_name: String; p_c_left, p_c_right: c_node);
      begin
        InHerited create_node(p_name);
        m_c_left:= p_c_left;
        m_c_right:= p_c_right;
      end; // create_c_operation_node

    begin // u_c_expression_tree
    end. // u_c_expression_tree



