// 005 u_c_instruction_list_15
// 11 dec 2004

// -- Delphi implementation from the book:
// --   Design Patterns Elements of Reusable Object-Oriented Software
// --   Addison Wesley - 1994 - ISBN 0-201-63361-2
// -- (C) Erich GAMMA, Richard HELM, Ralph JOHNSON, John VLISSIDES 1994

// -- translation from C to Delphi performed by
// --   Felix John COLIBRI  http://www.felix-colibri.com

(*$r+*)

unit u_c_instruction_list_15;
  interface
    uses u_c_basic_object
        , u_c_ast_tree_15
        ;

    type c_instruction= Class(c_basic_object)
                          m_c_next_instruction: c_instruction;

                          m_c_rhs_expression: c_node;

                          Constructor create_instruction(p_name: String; p_c_rhs_expression: c_node);

                          procedure display_instruction; Virtual;
                          function f_display_instruction: String; Virtual;

                          // procedure nil_members_and_free; Virtual; Abstract;
                          Destructor Destroy; Override;
                        end; // c_instruction

         c_instruction_list= Class(c_basic_object)
                               m_c_first_instruction, m_c_last_instruction: c_instruction;
                               m_instruction_count: Integer;

                               Constructor create_instruction_list(p_name: String);
                               function f_instruction_count: Integer;

                               procedure add_instruction(p_c_instruction: c_instruction);
                               procedure add_instruction_at_start(p_c_instruction: c_instruction);

                               procedure display_instruction_list;
                               // procedure evaluate(p_c_variable_list: c_variable_list);

                               Destructor Destroy; Override;
                             end; // c_instruction_list

  implementation
    uses SysUtils, u_c_display;

    // -- c_instruction

    Constructor c_instruction.create_instruction(p_name: String; p_c_rhs_expression: c_node);
      begin
        Inherited create_basic_object(p_name);
        m_c_rhs_expression:= p_c_rhs_expression;
      end; // create_instruction

    procedure c_instruction.display_instruction;
      begin
        display(m_name+ ':= ');
        m_c_rhs_expression.display_node;
      end; // display_instruction

    function c_instruction.f_display_instruction: String;
      begin
        Result:= m_name;
      end; // f_display_instruction

    Destructor c_instruction.Destroy;
      begin
        InHerited;
      end; // Destroy

    // -- c_instruction_list

    Constructor c_instruction_list.create_instruction_list(p_name: String);
      begin
        Inherited create_basic_object(p_name);
      end; // create_instruction_list

    function c_instruction_list.f_instruction_count: Integer;
      begin
        Result:= m_instruction_count;
      end; // f_instruction_count

    procedure c_instruction_list.add_instruction(p_c_instruction: c_instruction);
      begin
        if m_c_first_instruction= nil
          then m_c_first_instruction:= p_c_instruction
          else m_c_last_instruction.m_c_next_instruction:= p_c_instruction;
        m_c_last_instruction:= p_c_instruction;
        Inc(m_instruction_count);
      end; // add_instruction

    procedure c_instruction_list.add_instruction_at_start(p_c_instruction: c_instruction);
      begin
        p_c_instruction.m_c_next_instruction:= m_c_first_instruction;
        m_c_first_instruction:= p_c_instruction;
        if m_c_last_instruction= Nil
          then m_c_last_instruction:= m_c_first_instruction;
        Inc(m_instruction_count);
      end; // add_instruction_at_start

    procedure c_instruction_list.display_instruction_list;
      var l_c_instruction: c_instruction;
      begin
        display('LIST '+ m_name+ ' '+ IntToStr(m_instruction_count));
        l_c_instruction:= m_c_first_instruction;
        while l_c_instruction<> nil do
        begin
          l_c_instruction.display_instruction;
          l_c_instruction:= l_c_instruction.m_c_next_instruction;
        end; // while l_c_instruction
      end; // display_instruction_list
(*
    procedure c_instruction_list.evaluate(p_c_variable_list: c_variable_list);

      procedure evaluate_instruction(p_c_instruction: c_instruction);

        function f_evaluate_rhs_expression(p_c_expression: c_node): Integer;
          begin
            with p_c_expression do
            begin
              display('> expr '+ p_c_expression.m_name);
              case m_name[1] of
                '+' : begin
                        Result:= f_evaluate_rhs_expression(m_c_left)+ f_evaluate_rhs_expression(m_c_right)
                      end;
                '*' : Result:= f_evaluate_rhs_expression(m_c_left)* f_evaluate_rhs_expression(m_c_right)

                else
                  if p_c_expression is c_identifier_node
                    then begin
                        with p_c_variable_list do
                        begin
                          if f_index_of(p_c_expression.m_name)< 0
                            then add_variable(p_c_expression.m_name, 0);
                          Result:= f_find_by_variable(p_c_expression.m_name);
                        end;
                      end
                    else
                      if p_c_expression is c_litteral_node
                        then begin
                            Result:= StrToInt(m_name);
                          end
                        else display_bug_stop('not + * k var');
              end; // case
              display('< expr '+ p_c_expression.m_name);
            end; // with p_c_expression
          end; // f_evaluate_rhs_expression

        var l_lhs_name: String;
            l_lhs_index: Integer;
            l_rhs_value: Integer;

        begin // evaluate_instruction
          with p_c_instruction do
          begin
            display('> evaluate '+ m_name);
            l_lhs_name:= m_name;
            with p_c_variable_list do
            begin
              if f_index_of(l_lhs_name)< 0
                then add_variable(l_lhs_name, 0);
              l_lhs_index:= f_index_of(l_lhs_name);
            end;

            l_rhs_value:= f_evaluate_rhs_expression(m_c_rhs_expression);

            p_c_variable_list.set_value(l_lhs_index, l_rhs_value);
            display('< evaluate '+ m_name);
          end; // with p_c_instruction_name
        end; // evaluate_instruction

      var l_c_instruction: c_instruction;

      begin // evaluate
        display('Eval '+ m_name+ ' '+ IntToStr(m_instruction_count));

        l_c_instruction:= m_c_first_instruction;
        while l_c_instruction<> nil do
        begin
          evaluate_instruction(l_c_instruction);

          l_c_instruction:= l_c_instruction.m_c_next_instruction;
        end; // while l_c_instruction
      end; // evaluate
*)
    Destructor c_instruction_list.Destroy;
      var l_c_destroy: c_instruction;
      begin
        while m_c_first_instruction<> nil do
        begin
          l_c_destroy:= m_c_first_instruction;
          m_c_first_instruction:= m_c_first_instruction.m_c_next_instruction;
          l_c_destroy.Free;
        end;

        InHerited;
      end; // Destroy

    end.

