// 005 u_c_interpreter_15
// 11 dec 2004

// -- Delphi implementation from the book:
// --   Design Patterns Elements of Reusable Object-Oriented Software
// --   Addison Wesley - 1994 - ISBN 0-201-63361-2
// -- (C) Erich GAMMA, Richard HELM, Ralph JOHNSON, John VLISSIDES 1994

// -- translation from C to Delphi performed by
// --   Felix John COLIBRI  http://www.felix-colibri.com

(*$r+*)

unit u_c_interpreter_15;
  interface
    uses u_c_instruction_list_15, u_c_variable_list_15;

    type c_interpreter= class
                          Constructor create_interpreter;
                          procedure interpret(p_c_instruction_list: c_instruction_list;
                              p_c_variable_list: c_variable_list);
                          Destructor Destroy; Override;
                        end; // c_interpreter

  implementation
    uses SysUtils, u_c_display, u_c_ast_tree_15;

    // -- c_interpreter

    Constructor c_interpreter.create_interpreter;
      begin
        Inherited Create;
      end; // create_interpreter

    procedure c_interpreter.interpret(p_c_instruction_list: c_instruction_list;
        p_c_variable_list: c_variable_list);

      procedure evaluate_instruction(p_c_instruction: c_instruction);

        function f_evaluate_rhs_expression(p_c_expression: c_node): Integer;
          begin
            with p_c_expression do
            begin
              display('> expr '+ p_c_expression.m_name);
              case m_name[1] of
                '+' : begin
                        Result:= f_evaluate_rhs_expression(m_c_left)+ f_evaluate_rhs_expression(m_c_right)
                      end;
                '*' : Result:= f_evaluate_rhs_expression(m_c_left)* f_evaluate_rhs_expression(m_c_right)

                else
                  if p_c_expression is c_identifier_node
                    then begin
                        with p_c_variable_list do
                        begin
                          if f_index_of(p_c_expression.m_name)< 0
                            then add_variable(p_c_expression.m_name, 0);
                          Result:= f_find_by_variable(p_c_expression.m_name);
                        end;
                      end
                    else
                      if p_c_expression is c_litteral_node
                        then begin
                            Result:= StrToInt(m_name);
                          end
                        else display_bug_stop('not + * k var');
              end; // case
              display('< expr '+ p_c_expression.m_name);
            end; // with p_c_expression
          end; // f_evaluate_rhs_expression

        var l_lhs_name: String;
            l_lhs_index: Integer;
            l_rhs_value: Integer;

        begin // evaluate_instruction
          with p_c_instruction do
          begin
            display('> evaluate '+ m_name);
            l_lhs_name:= m_name;
            with p_c_variable_list do
            begin
              if f_index_of(l_lhs_name)< 0
                then add_variable(l_lhs_name, 0);
              l_lhs_index:= f_index_of(l_lhs_name);
            end; // with p_c_variable_list

            l_rhs_value:= f_evaluate_rhs_expression(m_c_rhs_expression);

            p_c_variable_list.set_value(l_lhs_index, l_rhs_value);
            display('< evaluate '+ m_name);
          end; // with p_c_instruction_name
        end; // evaluate_instruction

      var l_c_instruction: c_instruction;

      begin // interpret
        with p_c_instruction_list do
        begin
          display('interpret '+ IntToStr(m_instruction_count));

          l_c_instruction:= m_c_first_instruction;
          while l_c_instruction<> nil do
          begin
            evaluate_instruction(l_c_instruction);

            l_c_instruction:= l_c_instruction.m_c_next_instruction;
          end; // while l_c_instruction
        end; // with p_c_instruction_list  
      end; // interpret

    Destructor c_interpreter.Destroy;
      begin
        Inherited;
      end; // Destroy

    begin // u_c_interpreter
    end. // u_c_interpreter
