// 005 u_c_parser_15
// 11 dec 2004

// -- Delphi implementation from the book:
// --   Design Patterns Elements of Reusable Object-Oriented Software
// --   Addison Wesley - 1994 - ISBN 0-201-63361-2
// -- (C) Erich GAMMA, Richard HELM, Ralph JOHNSON, John VLISSIDES 1994

// -- translation from C to Delphi performed by
// --   Felix John COLIBRI  http://www.felix-colibri.com

(*$r+*)

unit u_c_parser_15;
  interface
    uses Classes, u_types_constants
        , u_c_basic_object
        , u_c_scanner_15
        , u_c_ast_tree_15
        , u_c_instruction_list_15
        ;

    type c_parser= class(c_basic_object)
                     constructor create_parser(p_name: String);

                     procedure parse(p_c_scanner: c_scanner; p_c_instruction_list: c_instruction_list);

                     destructor Destroy; Override;
                   end; // c_parser

  implementation
    uses SysUtils, Dialogs, u_c_display
        ;

    constructor c_parser.create_parser(p_name: String);
      begin
        Inherited create_basic_object(p_name);
      end; // create_parser

    procedure c_parser.parse(p_c_scanner: c_scanner; p_c_instruction_list: c_instruction_list);

      procedure analyse_assignment;

        function f_c_expression: c_node;

          function f_c_term: c_node;

            function f_c_factor: c_node;
              begin
                with p_c_scanner do
                begin
                  display('> factor '+ m_symbol_string);

                  case m_symbol_type of
                    e_identifier_symbol :
                        begin
                          Result:= c_identifier_node.create_identifier_node(m_symbol_string);
                          read_symbol;
                        end;
                    e_integer_symbol :
                        begin
                          Result:= c_litteral_node.create_litteral_node(m_symbol_string);
                          read_symbol;
                        end;
                    e_opening_parenthesis_symbol :
                        begin
                          read_symbol;

                          Result:= f_c_expression;

                          if m_symbol_type= e_closing_parenthesis_symbol
                            then read_symbol
                            else display_error('manque )');
                        end;
                    else display_error(concat('attend fact: k, var, (: '+ m_symbol_string));
                  end; // case

                  display('< factor '+ m_symbol_string);
                end; // with p_c_scanner
              end; // f_c_factor

            function f_c_handle_multiplication(p_symbol_string: String): c_node;
              var l_c_factor: c_node;
              begin
                with p_c_scanner do
                begin
                  display('> f_c_multiply '+ m_symbol_string);
                  if m_symbol_type in [e_times_symbol, e_slash_symbol]
                    then begin
                        read_symbol;
                        l_c_factor:= f_c_factor;
                        Result:= f_c_handle_multiplication(m_symbol_string);

                        // -- build the tree on the way back up
                        if Result<> Nil
                          then Result:= c_operation_node.create_operation_node(p_symbol_string, l_c_factor, Result)
                          else Result:= l_c_factor;
                      end
                    else Result:= Nil;
                  display('< f_c_multiply '+ m_symbol_string);
                end; // with p_c_scanner
              end; // f_c_handle_multiplication

            var l_save_symbol_string: String;
                l_c_multiply: c_node;

            begin // f_c_term
              with p_c_scanner do
              begin
                display('> f_c_term '+ m_symbol_string);
                Result:= f_c_factor;

                if m_symbol_type in [e_times_symbol, e_slash_symbol]
                  then begin
                      l_save_symbol_string:= m_symbol_string;
                      l_c_multiply:= f_c_handle_multiplication(m_symbol_string);
                      Result:= c_operation_node.create_operation_node(l_save_symbol_string, Result, l_c_multiply);
                    end;

                display('< f_c_term '+ m_symbol_string);
              end; // with p_c_scanner
            end; // f_c_term

          function f_c_handle_addition(p_symbol_string: String): c_node;
            var l_c_term: c_node;
            begin
              with p_c_scanner do
              begin
                display('> f_c_addition '+ m_symbol_string);
                if m_symbol_type in [e_plus_symbol, e_minus_symbol]
                  then begin
                      read_symbol;
                      l_c_term:= f_c_term;
                      Result:= f_c_handle_addition(m_symbol_string);

                      // -- build the tree on the way back up
                      if Result<> Nil
                        then Result:= c_operation_node.create_operation_node(p_symbol_string, l_c_term, Result)
                        else Result:= l_c_term;
                    end
                  else Result:= Nil;
                display('< f_c_addition '+ m_symbol_string);
              end; // with p_c_scanner
            end; // f_c_handle_addition

          function f_c_first_term: c_node;
            begin
              with p_c_scanner do
              begin
                // -- unary minus
                if m_symbol_type in [e_plus_symbol, e_minus_symbol]
                  then begin
  (*
                      // Result:= c_unary_operator_node.create_unary_operator_node(m_symbol_string);
                      Result:= c_unary_operator_node.create_unary_operator_node(m_symbol_string);
  *)
                      read_symbol;
                      display('unary');
                      Result:= c_operation_node.create_operation_node('-', f_c_term, Nil);
                    end
                  else Result:= f_c_term;
              end; // with p_c_scanner
            end; // f_c_first_term

          var l_c_add: c_node;
              l_save_symbol_string: String;

          begin // f_c_expression
            with p_c_scanner do
            begin
              display('> f_c_expression '+ m_symbol_string);

              Result:= f_c_first_term;

              if m_symbol_type in [e_plus_symbol, e_minus_symbol]
                then begin
                    l_save_symbol_string:= m_symbol_string;
                    l_c_add:= f_c_handle_addition(m_symbol_string);
                    Result:= c_operation_node.create_operation_node(l_save_symbol_string, Result, l_c_add);
                  end;

              display('< f_c_expression '+ m_symbol_string);
            end; // with p_c_scanner
          end; // f_c_expression
        var l_left_hand_identifier: String;
            l_c_expression_node: c_node;

        begin // analyze_assignemnt
          with p_c_scanner do
          begin
            display('> assignement '+ m_symbol_string);

            if m_symbol_type= e_identifier_symbol
              then begin
                  l_left_hand_identifier:= m_symbol_string;

                  read_symbol;
                  if m_symbol_type= e_becomes_symbol
                    then begin
                        read_symbol;

                        l_c_expression_node:= f_c_expression;

                        p_c_instruction_list.add_instruction(c_instruction.create_instruction(l_left_hand_identifier, l_c_expression_node));
                        if m_symbol_type= e_semi_colon_symbol
                          then read_symbol
                          else display_error('expects ;'); 
                      end
                    else display_error('"id := " := expected');
                end
              else display_error('"id := " id expected');

            display('< assignement');
          end; // with p_c_scanner
        end; // analyse_assignment

      begin // parse
        with p_c_scanner do
        begin
          display('> parse');

          read_symbol;
          while not f_end_of_text do
            analyse_assignment;
          display('< parse');
        end; // with p_c_scanner
      end; // parse

    destructor c_parser.Destroy;
      begin
        Inherited Destroy;
      end; // Destroy

    begin // u_c_parser
    end. // u_c_parser




