// 005 u_c_scanner_15
// 11 dec 2004

// -- Delphi implementation from the book:
// --   Design Patterns Elements of Reusable Object-Oriented Software
// --   Addison Wesley - 1994 - ISBN 0-201-63361-2
// -- (C) Erich GAMMA, Richard HELM, Ralph JOHNSON, John VLISSIDES 1994

// -- translation from C to Delphi performed by
// --   Felix John COLIBRI  http://www.felix-colibri.com

(*$r+*)

unit u_c_scanner_15;
  interface
    uses Classes, u_types_constants, u_c_basic_object
        , u_c_text_file;

    type t_symbol_type= (e_unknown_symbol,

                         e_identifier_symbol,

                         e_integer_symbol,

                         e_opening_parenthesis_symbol, e_closing_parenthesis_symbol,
                         e_plus_symbol, e_minus_symbol, e_slash_symbol, e_times_symbol,

                         e_semi_colon_symbol,
                         e_becomes_symbol,

                         e_end_symbol);

    type c_scanner= class(c_text_file)
                      m_symbol_type: t_symbol_type;
                      m_blank_string, m_symbol_string: String;
                      m_integer_value: Integer;

                      constructor create_scanner(p_name: String; p_full_file_name: String);

                      procedure display_error(p_text: String);

                      procedure read_symbol;

                      procedure test_scanner;

                      destructor Destroy; Override;
                    end; // c_scanner

  implementation
    uses SysUtils, Dialogs, u_c_display;

    // -- c_scanner

    constructor c_scanner.create_scanner(p_name: String; p_full_file_name: String);
      begin
        Inherited create_text_file(p_name, p_full_file_name);
      end; // create_scanner

    procedure c_scanner.display_error(p_text: String);
      begin
        ShowMessage(IntToStr(m_buffer_index)+ ' '+ p_text+ ' '+ m_symbol_string);
      end; // display_error

    procedure c_scanner.read_symbol;
      const k_blanks= [' ', chr(9)];
            k_returns= [chr(13), chr(10)];
            k_blanks_returns= k_blanks+ k_returns;

            k_digits= ['0'..'9'];
            k_identifier= ['a'..'z', 'A'..'Z', '_', '0'..'9'];

      procedure get_blanks;
        var l_start_index: Integer;
        begin
          l_start_index:= m_buffer_index;
          while (m_buffer_index< m_buffer_size) and (m_pt_buffer[m_buffer_index] in k_blanks_returns) do
            inc(m_buffer_index);
          m_blank_string:= f_extract_string_start_end(l_start_index, m_buffer_index- 1);
        end; // get_blanks

      procedure get_identifier;
        var l_start_index: Integer;
        begin
          l_start_index:= m_buffer_index;
          Inc(m_buffer_index);
          while (m_buffer_index< m_buffer_size) and (m_pt_buffer[m_buffer_index] in k_identifier) do
            inc(m_buffer_index);

          m_symbol_type:= e_identifier_symbol;
          m_symbol_string:= f_extract_string_start_end(l_start_index, m_buffer_index- 1);
        end; // get_identifier

      procedure get_number;
        var l_start_index: Integer;
        begin
          l_start_index:= m_buffer_index;
          while (m_buffer_index< m_buffer_size) and (m_pt_buffer[m_buffer_index] in k_digits) do
            inc(m_buffer_index);

          m_symbol_type:= e_integer_symbol;
          m_symbol_string:= f_extract_string_start_end(l_start_index, m_buffer_index- 1);
          m_integer_value:= StrToInt(m_symbol_string);
        end; // get_number

      procedure get_operator(p_operator_type: t_symbol_type);
        begin
          m_symbol_type:= p_operator_type;
          m_symbol_string:= m_pt_buffer[m_buffer_index];
          Inc(m_buffer_index);
        end; // get_operator

      begin // read_symbol
        get_blanks;

        if m_buffer_index< m_buffer_size
          then begin
              case m_pt_buffer[m_buffer_index] of
                'a'..'z', 'A'..'Z' : get_identifier;
                '0'..'9' : get_number;

                '(' : get_operator(e_opening_parenthesis_symbol);
                ')' : get_operator(e_closing_parenthesis_symbol);
                '+' : get_operator(e_plus_symbol);
                '-' : get_operator(e_minus_symbol);
                '*' : get_operator(e_times_symbol);
                '/' : get_operator(e_slash_symbol);
                ';' : get_operator(e_semi_colon_symbol);
                ':' : if (m_buffer_index+ 1< m_buffer_size)
                         and (m_pt_buffer[m_buffer_index+ 1]= '=')
                       then begin
                           m_symbol_string:= ':=';
                           m_symbol_type:= e_becomes_symbol;
                           Inc(m_buffer_index, 2);
                         end;
                else display_error('unknown character '+ m_pt_buffer[m_buffer_index]);
              end; // case
            end
          else begin
              m_symbol_type:= e_end_symbol;
              m_symbol_string:= '';
            end;
      end; // read_symbol

    procedure c_scanner.test_scanner;
      begin
        m_symbol_type:= e_unknown_symbol;

        while m_symbol_type<> e_end_symbol do
        begin
          read_symbol;
          display(m_symbol_string);
        end;
      end; // test_scanner

    destructor c_scanner.Destroy;
      begin
        Inherited Destroy;
      end; // Destroy

    begin // u_c_scanner
    end. // u_c_scanner


